'''
..
  $Id$
  
  GTK python tools of the clazzes.org project
  http://www.clazzes.org
 
  Created: 21.09.2018
 
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at
 
      http://www.apache.org/licenses/LICENSE-2.0
 
  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License. 
'''

import gi

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
from gi.repository import Gdk

import logging

log = logging.getLogger(__name__)

class StatusbarWithHistory(Gtk.EventBox):
    '''
       A variant of Gtk.Statusbar with a popup showing a message history.
       
       :ivar history_size: The size of the history being shown in the popup.
    '''
    
    def __init__(self,history_size=10):
        '''
           Constructor, which initializes the statusbar.
           
           :param history_size: The size of the history being shown in the popup.
        '''
        Gtk.EventBox.__init__(self)
        
        self.statusbar=Gtk.Statusbar()
        self.history_popup = None
        self.add_events(Gdk.EventType.BUTTON_PRESS)
        self.connect("button-press-event",self._show_history)
        self.statusbar.connect("text-pushed",self._on_text_pushed)
        self.history = []
        self.history_size = history_size
        
        self.get_context_id = self.statusbar.get_context_id
        self.push = self.statusbar.push
        self.pop = self.statusbar.pop
        self.remove = self.statusbar.remove
        self.remove_all = self.statusbar.remove_all
        self.get_message_area = self.statusbar.get_message_area
        self.add(self.statusbar)
        self.statusbar.connect("unmap",self._destroy_popup)
    
    def _forget_popup(self,*args): # @UnusedVariable
        
        if log.getEffectiveLevel() <= logging.DEBUG:
            log.debug("StatusBarWithHistory._forget_popup()")
            
        self.history_popup = None
    
    def _destroy_popup(self,*args):  # @UnusedVariable
        
        if log.getEffectiveLevel() <= logging.DEBUG:
            log.debug("StatusBarWithHistory._destroy_popup()")

        if self.history_popup is not None:
            self.history_popup.destroy()
            
        self.history_popup = None
    
    def _show_history(self,*args):  # @UnusedVariable
        
        self._destroy_popup()
        
        if len(self.history) == 0:
            return
        
        self.history_popup = Gtk.Popover()
        self.history_popup.set_constrain_to(Gtk.PopoverConstraint.WINDOW)
        self.history_popup.set_relative_to(self.statusbar)
        self.history_popup.set_position(Gtk.PositionType.TOP)
        
        vbox = Gtk.VBox()
        
        for msg in self.history:
            vbox.add(Gtk.Label(msg))

        self.history_popup.add(vbox)
        self.history_popup.show_all()
        self.history_popup.connect("closed",self._destroy_popup)
    
    def _on_text_pushed(self,statusbar,context_id,text):  # @UnusedVariable
        
        if self.history_popup is not None:
            vbox = self.history_popup.get_child()
        else:
            vbox = None

        while len(self.history) >= self.history_size:
            self.history.pop(0)
            if vbox is not None:
                vbox.remove(vbox.get_children()[0])
        
        self.history.append(text) 
        if vbox is not None:
            lbl = Gtk.Label(text)
            vbox.add(lbl)
            lbl.show()
        
        